/*
 * Copyright (c) 2008 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup net
 * @{
 */

/** @file
 */

#include <malloc.h>

#include "configuration.h"
#include "packet.h"
#include "packet_queue.h"

#define PQ_MAGIC_VALUE	0x23465968;

static inline int	pq_is_valid( pq_item_ref item );

pq_item_ref	pq_add( pq_head * queue, packet_t packet, int order, size_t metric ){
	pq_item_ref	item;
	pq_item_ref	tmp;

	tmp = pq_create( packet, order, metric );
	if( ! tmp ) return NULL;
	item = queue;
	while( pq_is_valid( item )){
		if( item->order < order ){
			if( item->next && pq_is_valid( item->next )){
				item = item->next;
			}else{
				item->next = tmp;
				tmp->previous = item;
				return tmp;
			}
		}else{
			tmp->previous = item->previous;
			tmp->next = item;
			item->previous = tmp;
			if( tmp->previous ){
				tmp->previous->next = tmp;
			}else{
				* queue = tmp;
			}
			return tmp;
		}
	}
}


pq_head	pq_create( packet_t packet, int order, size_t metric ){
	pq_item_ref	item;

	item = ( pq_item_ref ) malloc( sizeof( pq_item_t ));
	if( ! item ) return NULL;
	item->order = order;
	item->metric = metric;
	item->packet = packet;
	item->previous = NULL;
	item->next = NULL;
	item->magic_value = PQ_MAGIC_VALUE;
	return item;
}

void pq_destroy( pq_head queue ){
	pq_item_ref	actual;
	pq_item_ref	next;

	actual = queue;
	while( pq_is_valid( actual )){
		next = actual->next;
		actual->magic_value = 0;
		free( actual );
		actual = next;
	}
}

static inline int pq_is_valid( pq_item_ref item ){
	return item && ( item->magic_value == PQ_MAGIC_VALUE );
}

/** @}
 */
